@@ -5,7 +5,7 @@ import logging
 import traceback
 from abc import ABC
 from dataclasses import dataclass
-from typing import Union, Callable, Any, Type
+from typing import Union, Callable, Any, Type, ClassVar
 
 from async_tools import acall
 from dict_caster import DictCaster, Item
@@ -22,12 +22,12 @@ logger = logging.getLogger(__name__)
 @dataclass(frozen=True)
 class JsonBodyParameter(Parameter):
     in_: ParameterLocation = ParameterLocation.body
-    body_mime_type = "application/json"
+    body_mime_type: ClassVar[str] = "application/json"
 
 
 @dataclass(frozen=True)
 class RawBodyParameter(Parameter):
-    body_mime_type: str = "application/octet-stream"
+    body_mime_type: ClassVar[str] = "application/octet-stream"
     name: str = None
     schema: Type = bytes
     required: bool = True
