@@ -3,11 +3,11 @@ import datetime
 import functools
 import inspect
 import logging
-import typing
 from dataclasses import dataclass, field
 from decimal import Decimal
-from enum import Enum
-from typing import Optional, Type, Any, Union
+from enum import StrEnum
+from types import UnionType
+from typing import Optional, Type, Any, Union, ClassVar
 
 from apispec import APISpec
 from dict_caster.extras import first
@@ -17,7 +17,7 @@ from init_helpers.dict_to_dataclass import NoValue
 logger = logging.getLogger(__name__)
 
 
-class ParameterLocation(str, Enum):
+class ParameterLocation(StrEnum):
     query = 'query'
     path = 'path'
     header = 'header'
@@ -28,9 +28,9 @@ class ParameterLocation(str, Enum):
 @dataclass(frozen=True)
 class Parameter:
     name: str
-    schema: Type
+    schema: Type | UnionType
     in_: ParameterLocation
-    body_mime_type = None
+    body_mime_type: ClassVar[str] = None
     default: Any = NoValue
 
     @property
@@ -78,7 +78,7 @@ class RequestBody:
 
 
 @dataclass
-class SecuritySchemeType(str, Enum):
+class SecuritySchemeType(StrEnum):
     http = "http"
     api_key = "apiKey"
     oauth2 = "oauth2"
@@ -138,10 +138,8 @@ class OpenApiWrapper:
         self.context = context
         self.endpoints = []
 
-    @staticmethod
-    def _get_version() -> str:
-        now = datetime.datetime.now()
-        return f"{now.year - 2021}.{now.month}.{now.day}{now.hour:02d}{now.minute:02d}"
+    def register_endpoint(self, endpoint: Endpoint) -> None:
+        self.endpoints.append(endpoint)
 
     def get_spec_dict(self) -> dict:
         spec = APISpec(
@@ -154,15 +152,53 @@ class OpenApiWrapper:
         return spec.to_dict()
 
     @staticmethod
-    def _get_name_for_object(object_: Any) -> str:
-        return f"component_{object_.__name__}"
+    def _get_version() -> str:
+        now = datetime.datetime.now()
+        return f"{now.year - 2021}.{now.month}.{now.day}{now.hour:02d}{now.minute:02d}"
+
+    def _add_endpoint_to_spec(self, spec: APISpec, endpoint: Endpoint) -> None:
+        operation_dict = {
+            "summary": "",
+            "operationId": endpoint.operation_id,
+            "security": self._add_operation_securities(spec, endpoint.securities),
+            "parameters": self._add_operation_parameters(spec, endpoint.parameters),
+            "responses": self._add_operation_code_to_answer(spec, endpoint.code_to_answer),
+        }
+        if request_body_datum := self._add_operation_request_body(spec, endpoint.request_body):
+            operation_dict["requestBody"] = request_body_datum
+
+        spec.path(endpoint.path, operations={endpoint.method: operation_dict})
+
+    def _add_operation_parameters(self, spec: APISpec, parameters: list[Parameter]) -> list[dict[str, Any]]:
+        result: list[dict[str, Any]] = []
+        for param in parameters:
+            schema_name = self._add_schema_to_spec(spec, param.schema, default=param.default)
+            result.append({"name": param.name, "in": param.in_, "required": param.required, "schema": schema_name})
+        return result
+
+    def _add_operation_securities(self, spec: APISpec, securities: list[Security]) -> list[dict[str, list[str]]]:
+        return [{self._add_security_schema_to_spec(spec, security.scheme): security.scopes} for security in securities]
+
+    def _add_operation_code_to_answer(self, spec: APISpec, code_to_answer: dict[int, Type[Answer]]) -> dict[str, str]:
+        result: dict[str, str] = {}
+        for code, answer in code_to_answer.items():
+            response_name = self._add_answer_to_spec(spec, answer)
+            result[str(int(code))] = response_name
+        return result
+
+    def _add_operation_request_body(self, spec: APISpec, request_body: Optional[RequestBody]) -> dict[str, Any] | None:
+        result = None
+        if request_body is not None:
+            schema_name = self._add_schema_to_spec(spec, request_body.content.schema)
+            result = {"required": True, "content": {request_body.content.mime_type: {"schema": schema_name}}}
+        return result
 
     @functools.cache
-    def _add_schema_to_spec(self, spec: APISpec, schema: Type, default: Any = NoValue) -> Union[str, dict]:
+    def _add_schema_to_spec(self, spec: APISpec, schema: type | UnionType, default: Any = NoValue) -> Union[str, dict]:
         logger.debug('_add_schema_to_spec: %s', schema)
         optional = False
 
-        if getattr(schema, '__origin__', None) is typing.Union:
+        if getattr(schema, '__origin__', None) is Union:
             type_args = getattr(schema, '__args__', None)
             non_none_types = [type_ for type_ in type_args if type_ is not type(None)]
             if len(non_none_types) != 1:
@@ -218,14 +254,14 @@ class OpenApiWrapper:
         elif issubclass(schema, str):
             description = {"type": "string"}
         elif issubclass(schema, Decimal):
-            description = {"type": "string"}
+            description = {"type": "string", 'pattern': r'^\d*\.?\d*$'}
         else:
             raise TypeError(f"Unknown type: {schema}")
 
         if optional:
             description['nullable'] = optional
 
-        if default != NoValue and not callable(default) and not inspect.isawaitable(default):
+        if default is None or isinstance(default, (int, float, str, list, dict)):
             description['default'] = default
 
         return description
@@ -261,42 +297,6 @@ class OpenApiWrapper:
             })
         return security_schema.name
 
-    def _add_endpoint_to_spec(self, spec: APISpec, endpoint: Endpoint):
-        responses: dict[str, str] = {}
-        securities: list[dict] = []
-        parameters: list[dict] = []
-        operation_dict = {
-            "summary": "",
-            "operationId": endpoint.operation_id,
-            "security": securities,
-            "parameters": parameters,
-            "responses": responses,
-        }
-
-        if endpoint.request_body is not None:
-            schema_name = self._add_schema_to_spec(spec, endpoint.request_body.content.schema)
-            operation_dict["requestBody"] = {
-                "required": True,
-                "content": {
-                    endpoint.request_body.content.mime_type: {"schema": schema_name}
-                }
-            }
-
-        for parameter in endpoint.parameters:
-            schema_name = self._add_schema_to_spec(spec, parameter.schema, default=parameter.default)
-            parameters.append(
-                {"name": parameter.name, "in": parameter.in_, "required": parameter.required, "schema": schema_name}
-            )
-
-        for security in endpoint.securities:
-            security_scheme_name = self._add_security_schema_to_spec(spec, security.scheme)
-            securities.append({security_scheme_name: security.scopes})
-
-        for code, answer in endpoint.code_to_answer.items():
-            response_name = self._add_answer_to_spec(spec, answer)
-            responses[str(int(code))] = response_name
-
-        spec.path(endpoint.path, operations={endpoint.method: operation_dict})
-
-    def register_endpoint(self, endpoint: Endpoint):
-        self.endpoints.append(endpoint)
+    @staticmethod
+    def _get_name_for_object(object_: Any) -> str:
+        return f"component_{object_.__name__}"
